@@ -27,8 +27,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -55,8 +58,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -82,11 +94,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc,v 1.9 2004/03/12 01:59:04 fyodor Exp $ */
+/* $Id: TargetGroup.cc 3120 2006-02-07 07:15:32Z fyodor $ */
 
 #include "TargetGroup.h"
 #include "NmapOps.h"
@@ -106,6 +119,42 @@ void TargetGroup::Initialize() {
   ipsleft = 0;
 }
 
+/* take the object back to the begining without  (mdmcl)
+ * reinitalizing the data structures */  
+int  TargetGroup::rewind() {
+
+  /* For netmasks we must set the current address to the
+   * starting address and calculate the ips by distance */
+  if (targets_type == IPV4_NETMASK) {
+      currentaddr = startaddr;
+      if (startaddr.s_addr <= endaddr.s_addr) { 
+	ipsleft = endaddr.s_addr - startaddr.s_addr + 1;
+	return 0; 
+      }
+      else
+        assert(FALSE);
+  }
+  /* For ranges, we easily set current to zero and calculate
+   * the ips by the number of values in the columns */
+  else if (targets_type == IPV4_RANGES) {
+    memset((char *)current, 0, sizeof(current));
+    ipsleft = (last[0] + 1) * (last[1] + 1) *
+      (last[2] + 1) * (last[3] + 1);
+    return 0;
+  }
+#if HAVE_IPV6
+  /* For IPV6 there is only one address, this function doesn't
+   * make much sence for IPv6 does it? */
+  else if (targets_type == IPV6_ADDRESS) {
+    ipsleft = 1;
+    return 0;
+  }
+#endif 
+
+  /* If we got this far there must be an error, wrong type */
+  return -1;
+}
+
  /* Initializes (or reinitializes) the object with a new expression, such
     as 192.168.0.0/16 , 10.1.0-5.1-254 , or fe80::202:e3ff:fe14:1102 .  
     Returns 0 for success */  
@@ -130,7 +179,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     if (strchr(hostexp, ':'))
       fatal("Invalid host expression: %s -- colons only allowed in IPv6 addresses, and then you need the -6 switch", hostexp);
 
-    /*strauct in_addr current_in;*/
+    /*struct in_addr current_in;*/
     addy[0] = addy[1] = addy[2] = addy[3] = addy[4] = NULL;
     addy[0] = r = hostexp;
     /* First we break the expression up into the four parts of the IP address
@@ -138,8 +187,8 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     target_net = strtok(hostexp, "/");
     s = strtok(NULL, "");    /* find the end of the token from hostexp */
     netmask  = ( s ) ? atoi(s) : 32;
-    if ((int) netmask < 0 || netmask > 32) {
-      fprintf(stderr, "Illegal netmask value (%d), must be /0 - /32 .  Assuming /32 (one host)\n", netmask);
+    if ((int) netmask <= 0 || netmask > 32) {
+      fprintf(stderr, "Illegal netmask value (%d), must be /1 - /32 .  Assuming /32 (one host)\n", netmask);
       netmask = 32;
     }
     for(i=0; *(hostexp + i); i++) 
@@ -172,30 +221,24 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
       return 1;
     }
     else {
-      i=0;
       targets_type = IPV4_RANGES;
+      i=0;
+
       while(*++r) {
 	if (*r == '.' && ++i < 4) {
 	  *r = '\0';
 	  addy[i] = r + 1;
 	}
-	else if (*r == '[') {
-	  *r = '\0';
-	  addy[i]++;
-	}
-	else if (*r == ']') *r = '\0';
-	/*else if ((*r == '/' || *r == '\\') && i == 3) {
-	 *r = '\0';
-	 addy[4] = r + 1;
-	 }*/
-	else if (*r != '*' && *r != ',' && *r != '-' && !isdigit((int)*r)) fatal("Invalid character in  host specification.");
+	else if (*r != '*' && *r != ',' && *r != '-' && !isdigit((int)*r)) 
+	  fatal("Invalid character in  host specification.  Note in particular that square brackets [] are no longer allowed.  They were redundant and can simply be removed.");
       }
-      if (i != 3) fatal("Target host specification is illegal.");
+      if (i != 3) fatal("Invalid target host specification: %s", target_expr);
       
       for(i=0; i < 4; i++) {
 	j=0;
-	while((s = strchr(addy[i],','))) {
-	  *s = '\0';
+	do {
+	  s = strchr(addy[i],',');
+	  if (s) *s = '\0';
 	  if (*addy[i] == '*') { start = 0; end = 255; } 
 	  else if (*addy[i] == '-') {
 	    start = 0;
@@ -207,35 +250,20 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
 	    if ((r = strchr(addy[i],'-')) && *(r+1) ) end = atoi(r + 1);
 	    else if (r && !*(r+1)) end = 255;
 	  }
-	  if (o.debugging)
-	    log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end);
-	  if (start < 0 || start > end) fatal("Your host specifications are illegal!");
+	  /*	  if (o.debugging > 2)
+		  log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end); */
+	  if (start < 0 || start > end || start > 255 || end > 255)
+	    fatal("Your host specifications are illegal!");
+	  if (j + (end - start) > 255) 
+	    fatal("Your host specifications are illegal!");
 	  for(k=start; k <= end; k++)
 	    addresses[i][j++] = k;
-	  addy[i] = s + 1;
-	}
-	if (*addy[i] == '*') { start = 0; end = 255; } 
-	else if (*addy[i] == '-') {
-	  start = 0;
-	  if (!addy[i] + 1) end = 255;
-	  else end = atoi(addy[i]+ 1);
-	}
-	else {
-	  start = end = atoi(addy[i]);
-	  if ((r =  strchr(addy[i],'-')) && *(r+1) ) end = atoi(r+1);
-	  else if (r && !*(r+1)) end = 255;
-	}
-	if (o.debugging)
-	  log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end);
-	if (start < 0 || start > end) fatal("Your host specifications are illegal!");
-	if (j + (end - start) > 255) fatal("Your host specifications are illegal!");
-	for(k=start; k <= end; k++) 
-	  addresses[i][j++] = k;
-	last[i] = j - 1;
-	
+	  last[i] = j-1;
+	  if (s) addy[i] = s + 1;
+	} while (s);
       }
     }
-  memset((char *)current, 0, 4);
+  memset((char *)current, 0, sizeof(current));
   ipsleft = (last[0] + 1) * (last[1] + 1) *
     (last[2] + 1) * (last[3] + 1);
   }
@@ -253,7 +281,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     hints.ai_family = PF_INET6;
     rc = getaddrinfo(hostexp, NULL, &hints, &result);
     if (rc != 0) {
-      fprintf(stderr, "Failed to resolve given IPv6 hostname/IP: %s.  Note that you can't use '/mask' or '[1-4,7,100-]' style ranges for IPv6.  Error cod %d: %s\n", hostexp, rc, gai_strerror(rc));
+      fprintf(stderr, "Failed to resolve given IPv6 hostname/IP: %s.  Note that you can't use '/mask' or '[1-4,7,100-]' style ranges for IPv6.  Error code %d: %s\n", hostexp, rc, gai_strerror(rc));
       free(hostexp);
       if (result) freeaddrinfo(result);
       return 1;
@@ -272,8 +300,62 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
   return 0;
 }
 
+/* For ranges, skip all hosts in an octet,                  (mdmcl)
+ * get_next_host should be used for skipping the last octet :-) 
+ * returns: number of hosts skipped */
+int TargetGroup::skip_range(_octet_nums octet) {
+  unsigned long hosts_skipped = 0, /* number of hosts skipped */
+      oct = 0;           /* octect number */
+      int i = 0;                 /* simple lcv */
+
+  /* This function is only supported for RANGES! */
+  if (targets_type != IPV4_RANGES)
+    return -1;
+
+  switch (octet) {
+    case FIRST_OCTET:
+      oct = 0;
+      hosts_skipped = (last[1] + 1) * (last[2] + 1) * (last[3] + 1);
+      break;
+    case SECOND_OCTET:
+      oct = 1;
+      hosts_skipped = (last[2] + 1) * (last[3] + 1);
+      break;
+    case THIRD_OCTET:
+      oct = 2;
+      hosts_skipped = (last[3] + 1);
+      break;
+    default:  /* Hmm, how'd you do that */
+      return -1;
+  }
+
+  /* catch if we try to take more than are left */
+  assert(ipsleft + 1>= hosts_skipped);
+
+  /* increment the next octect that we can above us */
+  for (i = oct; i >= 0; i--) {
+    if (current[i] < last[i]) {
+      current[i]++;
+      break;
+    }
+    else
+      current[i] = 0;
+  }
+
+  /* reset all the ones below us to zero */
+  for (i = oct+1; i <= 3; i++) {
+    current[i] = 0;
+  }
+
+  /* we actauly don't skip the current, it was accounted for 
+   * by get_next_host */
+  ipsleft -= hosts_skipped - 1;
+ 
+  return hosts_skipped;
+}
+
  /* Grab the next host from this expression (if any) and uptdates its internal
-    state to reflect the the IP was given out.  Returns 0 and
+    state to reflect that the IP was given out.  Returns 0 and
     fills in ss if successful.  ss must point to a pre-allocated
     sockaddr_storage structure */
 int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
@@ -287,7 +369,7 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
   assert(sslen);
 
 
-  if (ipsleft <= 0)
+  if (ipsleft == 0)
     return -1;
   
   if (targets_type == IPV4_NETMASK) {
@@ -334,7 +416,7 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
       }
     }
     if (octet == -1) {
-      /* It didn't find anything to bump up, I muast have taken the last IP */
+      /* It didn't find anything to bump up, I must have taken the last IP */
       assert(ipsleft == 1);
       /* So I set current to last with the very final octet up one ... */
       /* Note that this may make current[3] == 256 */
@@ -359,7 +441,6 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
 #endif // HAVE_IPV6
   }
   ipsleft--;
-  assert(ipsleft >= 0);
   
   /* If we are resuming from a previous scan, we have already finished
      scans up to o.resume_ip.  */
@@ -405,7 +486,7 @@ int TargetGroup::return_last_host() {
 /* Lookahead is the number of hosts that can be
    checked (such as ping scanned) in advance.  Randomize causes each
    group of up to lookahead hosts to be internally shuffled around.
-   The target_expressions array MUST REMAIN VALID IN MEMMORY as long as
+   The target_expressions array MUST REMAIN VALID IN MEMORY as long as
    this class instance is used -- the array is NOT copied.
  */
 HostGroupState::HostGroupState(int lookahead, int rnd, 
