@@ -98,13 +98,16 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc 2947 2005-11-22 23:21:55Z fyodor $ */
+/* $Id: service_scan.cc 3024 2005-12-31 00:32:42Z fyodor $ */
 
 
 #include "service_scan.h"
 #include "timing.h"
 #include "NmapOps.h"
 #include "nsock.h"
+
+#include "tty.h"
+
 #if HAVE_OPENSSL
 #include <openssl/ssl.h>
 #endif
@@ -1655,7 +1658,7 @@ static void adjustPortStateIfNeccessary(ServiceNFO *svc) {
   char host[128];
 
   if (svc->port->state == PORT_OPENFILTERED) {
-    svc->port->state = PORT_OPEN;
+    svc->target->ports.addPort(svc->portno, svc->proto, NULL, PORT_OPEN);
 
     if (o.verbose || o.debugging > 1) {
       svc->target->NameIP(host, sizeof(host));
@@ -1809,6 +1812,14 @@ static int scanThroughTunnel(nsock_pool nsp, nsock_iod nsi, ServiceGroup *SG,
 
 /* Prints completion estimates and the like when appropriate */
 static void considerPrintingStats(ServiceGroup *SG) {
+   /* Check for status requests */
+   if (keyWasPressed()) {
+      SG->SPM->printStats(SG->services_finished.size() /
+                          ((double)SG->services_remaining.size() + SG->services_in_progress.size() + 
+                           SG->services_finished.size()), nsock_gettimeofday());
+   }
+
+
   /* Perhaps this should be made more complex, but I suppose it should be
      good enough for now. */
   if (SG->SPM->mayBePrinted(nsock_gettimeofday())) {
@@ -1951,6 +1962,14 @@ void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   SG = (ServiceGroup *) nsp_getud(nsp);
   nsi = nse_iod(nse);
 
+  // Check if a status message was requsted
+  if (keyWasPressed()) {
+     SG->SPM->printStats(SG->services_finished.size() /
+                         ((double)SG->services_remaining.size() + SG->services_in_progress.size() + 
+                          SG->services_finished.size()), nsock_gettimeofday());
+  }
+  
+
   if (svc->target->timedOut(nsock_gettimeofday())) {
     end_svcprobe(nsp, PROBESTATE_INCOMPLETE, SG, svc, nsi);
     return;
