@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.cc 2950 2005-11-28 01:34:08Z fyodor $ */
+/* $Id: tcpip.cc 3120 2006-02-07 07:15:32Z fyodor $ */
 
 #include <dnet.h>
 #include "tcpip.h"
@@ -875,8 +875,11 @@ else tcp->th_win = htons(1024 * (myttl % 4 + 1)); /* Who cares */
 tcp->th_sum = in_cksum((unsigned short *)pseudo, sizeof(struct tcphdr) + 
 		       optlen + sizeof(struct pseudo_header) + datalen);
 #endif
-/* Now for the ip header */
 
+if ( o.badsum )
+  --tcp->th_sum;
+
+/* Now for the ip header */
 memset(packet, 0, sizeof(struct ip)); 
 ip->ip_v = 4;
 ip->ip_hl = 5;
@@ -933,7 +936,7 @@ int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, unsigned int packetl
   struct ip *ip = (struct ip *) packet;
   struct tcphdr *tcp = NULL;
   udphdr_bsd *udp;
-  u8 *eth_frame;
+  u8 *eth_frame = NULL;
   eth_t *ethsd;
   bool ethsd_opened = false;
   assert(packet);
@@ -957,6 +960,8 @@ int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, unsigned int packetl
     PacketTrace::trace(PacketTrace::SENT, packet, packetlen); 
     if (ethsd_opened)
       eth_close(ethsd);
+    free(eth_frame);
+    eth_frame = NULL;
     return res;
   }
 
@@ -984,7 +989,7 @@ int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, unsigned int packetl
      must deal with it here rather than when building the packet,
      because they should be in NBO when I'm sending over raw
      ethernet */
-#if FREEBSD || BSDI || NETBSD || DEC
+#if FREEBSD || BSDI || NETBSD || DEC || MACOSX
   ip->ip_len = ntohs(ip->ip_len);
   ip->ip_off = ntohs(ip->ip_off);
 #endif
@@ -1093,6 +1098,9 @@ pingpkt.seq = seq;
 pingpkt.checksum = 0;
 pingpkt.checksum = in_cksum((unsigned short *)ping, icmplen);
 
+if ( o.badsum )
+  --pingpkt.checksum;
+
 return build_ip_raw(source, victim, o.ttl, IPPROTO_ICMP, get_random_u16(),
 		    ping, icmplen, packetlen);
 }
@@ -1294,6 +1302,9 @@ u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
   udp->uh_sum = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ + datalen);
 #endif
   
+  if ( o.badsum )
+    --udp->uh_sum;
+  
   /* Goodbye, pseudo header! */
   memset(pseudo, 0, sizeof(*pseudo));
   
@@ -1379,10 +1390,10 @@ ip->ip_id = htons(ipid);
 ip->ip_ttl = myttl;
 ip->ip_p = proto;
 ip->ip_src.s_addr = source->s_addr;
-#ifdef WIN32
+// #ifdef WIN32
 // TODO: Should this be removed? I'm not sure why this is here -- Fyodor
-if(source->s_addr == victim->s_addr) ip->ip_src.s_addr++;
-#endif
+// if(source->s_addr == victim->s_addr) ip->ip_src.s_addr++;
+// #endif
 ip->ip_dst.s_addr = victim->s_addr;
 #if HAVE_IP_IP_SUM
 ip->ip_sum = in_cksum((unsigned short *)ip, sizeof(struct ip));
@@ -2134,14 +2145,14 @@ int collect_dnet_interfaces(const struct intf_entry *entry, void *arg) {
    Strncpy(dcrn->ifaces[numifaces].devfullname, entry->intf_name, sizeof(dcrn->ifaces[numifaces].devfullname));
 
    /* Interface type */
-   if (entry->intf_type & INTF_TYPE_ETH) {
+   if (entry->intf_type == INTF_TYPE_ETH) {
 	   dcrn->ifaces[numifaces].device_type = devt_ethernet;
 	   /* Collect the MAC address since this is ethernet */
 	   memcpy(dcrn->ifaces[numifaces].mac, &entry->intf_link_addr.addr_eth.data, 6);
    }
-   else if (entry->intf_type & INTF_TYPE_LOOPBACK)
+   else if (entry->intf_type == INTF_TYPE_LOOPBACK)
 	   dcrn->ifaces[numifaces].device_type = devt_loopback;
-   else if (entry->intf_type & INTF_TYPE_TUN)
+   else if (entry->intf_type == INTF_TYPE_TUN)
 	   dcrn->ifaces[numifaces].device_type = devt_p2p;
    else dcrn->ifaces[numifaces].device_type = devt_other;
    
@@ -2296,7 +2307,7 @@ int sd;
 	eth_addr_t ethaddr;
 
 	if (!ethsd) 
-	  fatal("%s: Failed to open ethernet interface (%s)", __FUNCTION__,
+	  fatal("%s: Failed to open ethernet interface (%s). A possible cause on BSD operating systems is running out of BPF devices (see http://seclists.org/lists/nmap-dev/2006/Jan-Mar/0014.html).", __FUNCTION__,
 		mydevs[numifaces].devname);
 	if (eth_get(ethsd, &ethaddr) != 0) 
 	  fatal("%s: Failed to obtain MAC address for ethernet interface (%s)",
@@ -2571,7 +2582,22 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
       ifsin = (struct sockaddr_in *) &(iface->addr);
       if ((ifsin->sin_addr.s_addr & mask) == (dstsin->sin_addr.s_addr & mask))
 	rnfo->direct_connect = 1;
-      else rnfo->direct_connect = 0;
+      else {
+	rnfo->direct_connect = 0;
+	/* must find the next hop by checking route table ... */
+	routes = getsysroutes(&numroutes);
+	/* Now we simply go through the list and take the first match */
+	for(i=0; i < numroutes; i++) {
+	  if (strcmp(routes[i].device->devname, iface->devname) == 0 && 
+	      (routes[i].dest & routes[i].netmask) == 
+	      (dstsin->sin_addr.s_addr & routes[i].netmask)) {
+	    /* Yay, found a matching route. */
+	    ifsin = (struct sockaddr_in *) &rnfo->nexthop;
+	    ifsin->sin_family = AF_INET;
+	    ifsin->sin_addr.s_addr = routes[i].gw.s_addr;
+	  }
+	}
+      }
       memcpy(&rnfo->ii, iface, sizeof(rnfo->ii));
       if (o.spoofsource)
 	memcpy(&rnfo->srcaddr, &spoofss, sizeof(rnfo->srcaddr));
